#ifndef PW_SCENE_NODE_HPP
#define PW_SCENE_NODE_HPP

#include <pw/core/globals.hpp>

#include <string>
#include <vector>

namespace pw {

class component;

class node {
public:

    typedef shared_ptr<node> ref;
    typedef std::vector<ref> ref_array;
    typedef std::vector<node*> ptr_array;

    //! standard c'tor
    node(const std::string& name = "");

    //! copy c'tor
    node(const node& node);

    //! cloning interface
    virtual node* clone(const unsigned short& copymode) const;

    std::string name() const;
    void set_name(const std::string &name);

    //! return list of parents
    inline const ptr_array& parents() const { return _parents; }

    //! check if this is the root node
    inline bool is_root() const { return parents().empty(); }

    //! add a child node
    void add_child(node *anode);

    //! get the list of children
    inline const ref_array& children() const { return _children; }

    //! get number of children
    inline size_t child_count() const { return children().size(); }

    //! check if this node is a leaf node
    inline bool is_leaf() const { return children().empty(); }

    virtual ~node();

protected:

    ref_array _children;
    ptr_array _parents;

    std::string _name;
};

}

#endif
