#include "pw/scripting/script.hpp"

#include "pw/scripting/scripting.hpp"

#include "pw/core/vector.hpp"
#include "pw/core/quaternion.hpp"
#include "pw/core/axisangle.hpp"


struct lua_state : public pw::script::state {

    pw::scripting::state _state;
    pw::scripting::table _namespace;

    void load_modules();

    lua_state() {

        // create global pw namespace
        _namespace = _state.create_named_table("pw");

        // add the script as a user type
        _namespace.new_usertype<lua_state>("script",
                                           "initialize",&lua_state::load_modules
                                           );

        // add this instance as "script"
        _namespace.set("script",this);
    }

    virtual int run(const std::string &s);

};

int lua_state::run(const std::string &s)
{
    return _state.script(s.c_str()).get<int>();
}

void lua_state::load_modules() {

    // open all libraries
    _state.open_libraries();

    typedef double Scalar;

    _namespace.set("pi",pw::Pi<Scalar>());

    using namespace pw;

    _namespace.new_usertype<vector3d>("vector3",
                                      "set",&vector3d::set,
                                      "x", scripting::property(scripting::resolve<const Scalar&() const>(&vector3d::x), &vector3d::set_x),
                                      "y", scripting::property(scripting::resolve<const Scalar&() const>(&vector3d::y), &vector3d::set_y),
                                      "z", scripting::property(scripting::resolve<const Scalar&() const>(&vector3d::z), &vector3d::set_z),
                                      "v",&vector3d::values,
                                      "clone",&vector3d::clone
                                      );

    _namespace.new_usertype<quaterniond>("quaternion",
                                         "set",&quaterniond::set,
                                         "x", scripting::property(scripting::resolve<const Scalar&() const>(&quaterniond::x), &quaterniond::set_x),
                                         "y", scripting::property(scripting::resolve<const Scalar&() const>(&quaterniond::y), &quaterniond::set_y),
                                         "z", scripting::property(scripting::resolve<const Scalar&() const>(&quaterniond::z), &quaterniond::set_z),
                                         "w", scripting::property(scripting::resolve<const Scalar&() const>(&quaterniond::w), &quaterniond::set_w)
                                         //                                      "v",&vector3d::values,
                                         //                                      "clone",&vector3d::clone
                                         );

    _namespace.new_usertype<axisangled>("axisangle",
                                        "axis",scripting::property(&axisangled::axis,&axisangled::set_axis),
                                        "angle",scripting::property(&axisangled::angle,&axisangled::set_angle)
                                        );


}


pw::script::script()
{
    _state.reset(new lua_state());
}

pw::script::~script()
{
    _state = nullptr;
}

int pw::script::run(const std::string &s)
{
    return _state->run(s);

}




